@@ -1,13 +1,13 @@
 import warnings
+from collections.abc import Callable
 from copy import deepcopy
 from datetime import datetime
 from typing import Any, Literal, TypeVar
-from collections.abc import Callable
 
 from mmar_mapi.models.chat_item import ChatItem, ReplicaItem, OuterContextItem
 from mmar_mapi.models.widget import Widget
 from mmar_mapi.type_union import TypeUnion
-from pydantic import Field, ValidationError, Extra
+from pydantic import Field, ValidationError
 
 from .base import Base
 
@@ -146,10 +146,6 @@ class BaseMessage(Base):
     def widget(self) -> Widget | None:
         return _get_widget(self.content)
 
-    @staticmethod
-    def DATETIME_FORMAT() -> str:
-        return _DT_FORMAT
-
     def with_now_datetime(self):
         return self.model_copy(update=dict(date_time=now_pretty()))
 
@@ -161,6 +157,21 @@ class BaseMessage(Base):
     def is_human(self):
         return self.type == "human"
 
+    @staticmethod
+    def DATETIME_FORMAT() -> str:
+        return _DT_FORMAT
+
+    @staticmethod
+    def find_resource_id(msg: "BaseMessage", ext: str | None = None, type: str=None) -> str | None:
+        resource_id = msg.resource_id
+        if type and type != msg.type:
+            return None
+        if not resource_id:
+            return None
+        if ext and not resource_id.endswith(ext):
+            return None
+        return resource_id
+
 
 class HumanMessage(BaseMessage):
     type: Literal["human"] = "human"
@@ -182,15 +193,18 @@ class MiscMessage(BaseMessage):
 ChatMessage = TypeUnion[HumanMessage, AIMessage, MiscMessage]
 
 
+def find_in_messages(messages: list[ChatMessage], func: Callable[[ChatMessage], T | None]) -> T | None:
+    return next(filter(None, map(func, messages)), None)
+
+
 class Chat(Base):
     context: Context = Field(default_factory=Context)
     messages: list[ChatMessage] = Field(default_factory=list)
 
-    class Config:
-        extra = Extra.ignore
+    model_config = {"extra": "ignore"}
 
     def __init__(self, **data):
-        extra_fields = set(data.keys()) - set(self.__fields__.keys())
+        extra_fields = set(data.keys()) - set(type(self).model_fields.keys())
         if extra_fields:
             warnings.warn(f"Chat initialization: extra fields will be ignored: {extra_fields}")
         super().__init__(**data)
@@ -222,6 +236,12 @@ class Chat(Base):
                 return message.state
         return default
 
+    def find_in_messages(self, func: Callable[[ChatMessage], T | None]) -> T | None:
+        return find_in_messages(self.messages, func)
+
+    def rfind_in_messages(self, func: Callable[[ChatMessage], T | None]) -> T | None:
+        return find_in_messages(self.messages[::-1], func)
+
 
 def make_content(
     text: str | None = None,
@@ -229,11 +249,13 @@ def make_content(
     resource_id: str | None = None,
     command: dict | None = None,
     widget: Widget | None = None,
+    content: Content | None = None,
 ) -> Content:
     resource_id = (resource_id or None) and {"type": "resource_id", "resource_id": resource_id}
     command = (command or None) and {"type": "command", "command": command}
 
-    content = list(filter(None, [text, resource_id, command, widget]))
+    content = content if isinstance(content, list) else [content] if content else []
+    content += list(filter(None, [text, resource_id, command, widget]))
     if len(content) == 0:
         content = ""
     elif len(content) == 1:
