@@ -1,5 +1,6 @@
 from typing import Self, Literal
 
+from more_itertools import chunked
 from pydantic import BaseModel, model_validator
 
 
@@ -20,3 +21,14 @@ class Widget(BaseModel):
                     continue
                 raise ValueError(f"Expected buttons like `<callback>:<caption>`, found: {btn}")
         return self
+
+    @staticmethod
+    def make_inline_buttons(ibuttons: dict[str, str], by=1) -> "Widget":
+        return _make_inline_buttons(ibuttons=ibuttons, by=by)
+
+
+def _make_inline_buttons(ibuttons: dict[str, str], by=1) -> "Widget":
+    ibs0 = [f"{key}:{val}" for key, val in ibuttons.items()]
+    ibs = list(chunked(ibs0, n=by))
+    res = Widget(ibuttons=ibs)
+    return res
